# TAPAS

This is the main GitHub project for the implementation of the TAPAS application from Group 1.

## Project Structure

This project is structured as follows:

-   [.deployment/docker-compose.yml](.deployment/docker-compose.yml): Docker Compose configuration file for all services (deployment)
-   [.experiments](.experiments): Experiment files for chaos monkey tests
-   [.github/workflows](.github/workflows): GitHub actions scripts (CI/CD workflow)
-   [common](common): common library for shared elements across the whole application
    -   [common/src](common/src): source code of the library
    -   [common/pom.xml](common/pom.xml): Maven pom-file
-   [doc/architecture/decisions](doc/architecture/decisions): ADRs
-   [doc/workflow.png](doc/workflow.png): Workflow diagram
-   [executor-base](executor-base): library for the executors. Includes the logic for executors to connect to the system
    -   [executor-base/src](executor-base/src): source code of the library
    -   [executor-base/pom.xml](executor-base/pom.xml): Maven pom-file
-   [executor-computation](executor-computation): standalone project for the computation executor micro-service (Spring Boot project)
    -   [executor-computation/src](executor-computation/src): source code of the project
    -   [executor-computation/pom.xml](executor-computation/pom.xml): Maven pom-file
-   [executor-humidity](executor-humidity): standalone project for the humidity executor micro-service (Spring Boot project)
    -   [executor-humidity/src](executor-humidity/src): source code of the project
    -   [executor-humidity/pom.xml](executor-humidity/pom.xml): Maven pom-file
-   [executor-pool](executor-pool): standalone project for the executor-pool micro-service (Spring Boot project)
    -   [executor-pool/src](executor-pool/src): source code of the project (following the Hexagonal Architecture)
    -   [executor-pool/pom.xml](executor-pool/pom.xml): Maven pom-file
-   [executor-robot](executor-robot): standalone project for the robot executor micro-service (Spring Boot project)

    -   [executor-robot/src](executor-robot/src): source code of the project
    -   [executor-robot/pom.xml](executor-robot/pom.xml): Maven pom-file

-   [mocks](mocks): some auction-house mock files to test localy

-   [roster](roster): standalone project for the Roster micro-service (Spring Boot project)

    -   [roster/src](roster/src): source code of the project (following the Hexagonal Architecture)
    -   [roster/pom.xml](roster/pom.xml): Maven pom-file

-   [tapas-auction-house](tapas-auction-house): standalone project for the Tapas-Aution-House micro-service (Spring Boot project)

    -   [tapas-auction-house/src](tapas-auction-house/src): source code of the project (following the Hexagonal Architecture)
    -   [tapas-auction-house/pom.xml](tapas-auction-house/pom.xml): Maven pom-file

-   [tapas-tasks](tapas-tasks): standalone project for the Tapas-Tasks micro-service (Spring Boot project)

    -   [tapas-tasks/src](tapas-tasks/src): source code of the project (following the Hexagonal Architecture)
    -   [tapas-tasks/pom.xml](tapas-tasks/pom.xml): Maven pom-file

-   [docker-compose.yml](docker-compose.yml): Docker Compose configuration file for local development

## VM Configurations

Specs (we can upgrade if needed):

-   1 CPU
-   2 GB RAM
-   20 GB HD
-   Ubuntu 20.04

| Name               | Server IP     |
| ------------------ | ------------- |
| SCS-ASSE-VM-Group1 | 86.119.35.40  |
| SCS-ASSE-VM-Group2 | 86.119.35.213 |
| SCS-ASSE-VM-Group3 | 86.119.34.242 |
| SCS-ASSE-VM-Group4 | 86.119.35.199 |
| SCS-ASSE-VM-Group5 | 86.119.35.72  |
