package ch.unisg.executorbase;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import ch.unisg.common.valueobject.ExecutorURI;
import ch.unisg.executorbase.services.GetAssignmentService;
import ch.unisg.executorbase.services.NotifyExecutorPoolService;
import lombok.Getter;

@Component
public class Executor {

    @Getter
    ExecutorStatus executorStatus = ExecutorStatus.STARTING_UP;

    @Getter
    @Value("${executor.type}")
    String executorType;

    @Getter
    @Value("${executor.uri}")
    ExecutorURI executorUri;

    @Autowired
    NotifyExecutorPoolService notifyExecutorPoolService;

    @Autowired
    GetAssignmentService getAssignmentService;

    private Logger logger = Logger.getLogger(Executor.class.getName());

    public Executor() {
        executorStatus = ExecutorStatus.IDLING;
    }

    public void init() {
       if(!notifyExecutorPoolService.executorStarted(this.executorUri, this.executorType)) {
            logger.log(Level.WARNING, "ExecutorBase | Executor could not connect to executor pool! Shuting down!");
            System.exit(0);
        } else {
            logger.info("ExecutorBase | Executor conntected to executor pool");
            this.setIdling();
            getAssignmentService.getAssignment();
        }
    }

    // @PreDestroy
    // public void preDestroy() {
    //     System.out.println("TEST");
    //     notifyExecutorPoolService.executorStopped(this.executorUri);
    // }

    public void setIdling() {
        this.executorStatus = ExecutorStatus.IDLING;
    }

    public void setExecuting() {
        this.executorStatus = ExecutorStatus.EXECUTING;
    }

}
