package ch.unisg.executorbase.services;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

@Component
public class ExecutionFinishedService {

    @Value("${roster.uri}")
    private String rosterUri;

    private final Logger logger = Logger.getLogger(ExecutionFinishedService.class.getName());

    /**
    *   Publishes the execution finished event
    *   @return void
    **/
    public void publishExecutionFinishedEvent(String taskID, String outputData, String status) {

        logger.log(Level.INFO, "ExecutorBase | Sending finish execution event....");

        String body = new JSONObject()
        .put("taskID", taskID)
        .put("outputData", outputData)
        .put("status", status)
        .toString();

        HttpClient client = HttpClient.newHttpClient();
        HttpRequest request = HttpRequest.newBuilder()
                .uri(URI.create(rosterUri+"/task/completed"))
                .header("Content-Type", "application/json")
                .POST(HttpRequest.BodyPublishers.ofString(body))
                .build();


        try {
            client.send(request, HttpResponse.BodyHandlers.ofString());
        } catch (InterruptedException e) {
            logger.log(Level.SEVERE, e.getLocalizedMessage(), e);
            Thread.currentThread().interrupt();
        } catch (IOException e) {
            logger.log(Level.SEVERE, e.getLocalizedMessage(), e);
        }

        logger.log(Level.INFO, "ExecutorBase | Finish execution event sent with result: {0}", outputData);
    }

}
