package ch.unisg.executorbase.services;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Primary;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;

import ch.unisg.executorbase.Executor;
import ch.unisg.executorbase.Task;

@Component
@Primary
public class GetAssignmentService {

    @Value("${roster.uri}")
    String rosterUri;

    Logger logger = Logger.getLogger(GetAssignmentService.class.getName());

    @Autowired
    private Executor executor;

    @Autowired
    private ExecuteTaskServiceInterface executeTaskService;

    public void getAssignment() {
        Task newTask = requestAssignment();
        if (newTask != null) {
            logger.info("ExecutorBase | Executor got a new task");
            executeTaskService.executeTask(newTask);
        } else {
            logger.info("ExecutorBase | Executor got no new task");
            executor.setIdling();
        }
    }

    /**
    *   Requests a new task assignment
    *   @return the assigned task
    *   @see Task
    **/
    private Task requestAssignment() {

        String body = new JSONObject()
                .put("executorType", executor.getExecutorType())
                .put("executorURI", executor.getExecutorUri().getValue())
                .toString();

        HttpClient client = HttpClient.newHttpClient();
        HttpRequest request = HttpRequest.newBuilder()
                .uri(URI.create(rosterUri + "/task/apply"))
                .header("Content-Type", "application/json")
                .POST(HttpRequest.BodyPublishers.ofString(body))
                .build();

        try {
            logger.info("ExecutorBase | Sending getAssignment request");
            HttpResponse<String> response = client.send(request, HttpResponse.BodyHandlers.ofString());
            if (response.statusCode() != HttpStatus.OK.value()) {
                logger.info("ExecutorBase | No task assigned");
                return null;
            }
            logger.info("ExecutorBase | Task assigned");
            JSONObject responseBody = new JSONObject(response.body());

            if (!responseBody.get("inputData").equals(null)) {
                return new Task(responseBody.getString("taskID"), responseBody.getString("inputData"));
            }
            return new Task(responseBody.getString("taskID"));

        } catch (InterruptedException e) {
            logger.log(Level.SEVERE, e.getLocalizedMessage(), e);
            Thread.currentThread().interrupt();
        } catch (IOException e) {
            logger.log(Level.SEVERE, e.getLocalizedMessage(), e);
        }

        return null;
    }
}
