package ch.unisg.executorbase.services;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Primary;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;

import ch.unisg.common.valueobject.ExecutorURI;

@Component
@Primary
public class NotifyExecutorPoolService {

    @Value("${executor.pool.uri}")
    String executorPoolUri;

    Logger logger = Logger.getLogger(NotifyExecutorPoolService.class.getName());

    /**
    *   Notifies the executor-pool about the startup of this executor
    *   @return if the notification was successful
    **/
	public boolean executorStarted(ExecutorURI executorURI, String executorType) {

        String body = new JSONObject()
            .put("executorTaskType", executorType)
            .put("executorUri", executorURI.getValue())
            .toString();

        HttpClient client = HttpClient.newHttpClient();
        HttpRequest request = HttpRequest.newBuilder()
                .uri(URI.create(executorPoolUri + "/executor-pool/executors"))
                .header("Content-Type", "application/json")
                .POST(HttpRequest.BodyPublishers.ofString(body))
                .build();

        try {
            HttpResponse<String> response = client.send(request, HttpResponse.BodyHandlers.ofString());
            if (response.statusCode() == HttpStatus.CREATED.value()) {
                return true;
            }
        } catch (InterruptedException e) {
            logger.log(Level.SEVERE, e.getLocalizedMessage(), e);
            Thread.currentThread().interrupt();
        } catch (IOException e) {
            logger.log(Level.SEVERE, e.getLocalizedMessage(), e);
        }

         return false;
	}

    /**
    *   Notifies the executor-pool about the shutdown of this executor
    **/
	public static void executorStopped(String executorURI) {
        System.out.println("TEST");
        HttpClient client = HttpClient.newHttpClient();
        HttpRequest request = HttpRequest.newBuilder()
                .uri(URI.create("http://localhost:8083" + "/executor-pool/executors/" + executorURI))
                // .uri(URI.create(executorPoolUri + "/executor-pool/executors/" + executorURI))
                .header("Content-Type", "application/json")
                .DELETE()
                .build();

        try {
            client.send(request, HttpResponse.BodyHandlers.ofString());
        } catch (InterruptedException e) {
            // logger.log(Level.SEVERE, e.getLocalizedMessage(), e);
            // Thread.currentThread().interrupt();
        } catch (IOException e) {
            // logger.log(Level.SEVERE, e.getLocalizedMessage(), e);
        }
    }
}
