package ch.unisg.executorhumidity;

import java.io.IOException;
import java.io.StringReader;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.Map;
import java.util.Optional;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import ch.unisg.ics.interactions.wot.td.ThingDescription;
import ch.unisg.ics.interactions.wot.td.affordances.Form;
import ch.unisg.ics.interactions.wot.td.affordances.PropertyAffordance;
import ch.unisg.ics.interactions.wot.td.clients.TDCoapRequest;
import ch.unisg.ics.interactions.wot.td.clients.TDCoapResponse;
import ch.unisg.ics.interactions.wot.td.io.TDGraphReader;
import ch.unisg.ics.interactions.wot.td.schemas.ObjectSchema;
import ch.unisg.ics.interactions.wot.td.vocabularies.TD;

@Component
public class GetHumidityService {

    @Value("${search.engine.uri}")
    String endpoint;

    public String getHumidity() {

        String input = "@prefix dct: <http://purl.org/dc/terms/> . select ?title where { ?title dct:title 'Mirogate' }";

        var httpRequest = HttpRequest.newBuilder()
            .uri(URI.create(endpoint))
            .header("Content-Type", "application/sparql-query")
            .POST(HttpRequest.BodyPublishers.ofString(input))
            .build();

        var client = HttpClient.newHttpClient();

        try {
            String description = client.send(httpRequest, HttpResponse.BodyHandlers.ofString()).body();
            String mirogateUri = null;

            DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
            Document doc = dBuilder.parse(new InputSource(new StringReader(description)));

            doc.getDocumentElement().normalize();

            NodeList results = doc.getElementsByTagName("uri");
            for (int temp = 0; temp < results.getLength(); temp += 1) {
                Node nNode = results.item(temp);
                if (nNode.getTextContent().contains("mirogate")) {
                    mirogateUri = nNode.getTextContent();
                }
            }
            if (mirogateUri == null) {
                // TODO implement logic if execution failed
                return "";
            }

            // Parse a TD from a string
            ThingDescription td = TDGraphReader.readFromURL(ThingDescription.TDFormat.RDF_TURTLE, mirogateUri);

            Optional<PropertyAffordance> humidity = td.getPropertyByName("humidity");

            if (humidity.isPresent()) {
                Optional<Form> form = humidity.get().getFirstFormForOperationType(TD.readProperty);

                if (form.isPresent()) {
                    TDCoapRequest request = new TDCoapRequest(form.get(), TD.readProperty);

                    TDCoapResponse response = request.execute();
                    Map<String, Object> payload = response.getPayloadAsObject((ObjectSchema) humidity.get().getDataSchema());
                    Object result = payload.get("https://interactions.ics.unisg.ch/mirogate#HumidityValue");
                    return result.toString();

                }
            }

        } catch (IOException e) {
            e.printStackTrace();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (ParserConfigurationException e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        } catch (SAXException e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        }
        return null;
    }
}
