package ch.unisg.roster.roster.adapter.out.web;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Component;

import ch.unisg.common.valueobject.ExecutorURI;
import ch.unisg.roster.roster.application.port.out.NewTaskEventPort;
import ch.unisg.roster.roster.domain.ExecutorRegistry;
import ch.unisg.roster.roster.domain.event.NewTaskEvent;

@Component
@Primary
public class PublishNewTaskEventAdapter implements NewTaskEventPort {

    Logger logger = Logger.getLogger(PublishNewTaskEventAdapter.class.getName());

    /**
    *   Informs executors about the availability of a new task.
    *   @return void
    **/
    @Override
    public void publishNewTaskEvent(NewTaskEvent event) {

        Set<ExecutorURI> executors = ExecutorRegistry.getInstance().getExecutorsByType(event.taskType.getValue());

        for (ExecutorURI uri : executors) {
            HttpClient client = HttpClient.newHttpClient();
            HttpRequest request = HttpRequest.newBuilder()
                    .uri(URI.create(uri.getValue() + "/newtask/" + event.taskType.getValue()))
                    .GET()
                    .build();


            try {
                client.send(request, HttpResponse.BodyHandlers.ofString());
            } catch (InterruptedException e) {
                logger.log(Level.SEVERE, e.getLocalizedMessage(), e);
                Thread.currentThread().interrupt();
            } catch (IOException e) {
                logger.log(Level.SEVERE, e.getLocalizedMessage(), e);
            }
        }

    }

}
