package ch.unisg.roster.roster.adapter.out.web;

import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Component;

import ch.unisg.roster.roster.application.port.out.TaskAssignedEventPort;
import ch.unisg.roster.roster.domain.event.TaskAssignedEvent;

@Component
@Primary
public class PublishTaskAssignedEventAdapter implements TaskAssignedEventPort {

    @Value("${task.list.uri}")
    private String server;

    Logger logger = Logger.getLogger(PublishTaskAssignedEventAdapter.class.getName());

    /**
    *   Informs the task service about the assignment of the task.
    *   @return void
    **/
    @Override
    public void publishTaskAssignedEvent(TaskAssignedEvent event) {

        try {

            JSONObject op1 = new JSONObject()
            .put("op", "replace")
            .put("path", "/taskStatus")
            .put("value", "ASSIGNED");

            String body = new JSONArray().put(op1).toString();

            HttpClient client = HttpClient.newHttpClient();
            HttpRequest request = HttpRequest.newBuilder()
                .uri(URI.create(server + "/tasks/" + event.taskID))
                .header("Content-Type", "application/json-patch+json")
                .method("PATCH", HttpRequest.BodyPublishers.ofString(body))
                .build();

            client.send(request, HttpResponse.BodyHandlers.ofString());
        } catch (InterruptedException e) {
            logger.log(Level.SEVERE, e.getLocalizedMessage(), e);
            Thread.currentThread().interrupt();
        } catch (IOException e) {
            logger.log(Level.SEVERE, e.getLocalizedMessage(), e);
        }

    }

}
