package ch.unisg.roster.roster;


import ch.unisg.roster.roster.application.port.in.AddRosterItemPort;
import ch.unisg.roster.roster.domain.Roster;
import ch.unisg.roster.roster.domain.Task;
import ch.unisg.roster.roster.domain.valueobject.ExecutorType;
import org.json.JSONObject;
import org.json.JSONException;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.http.*;

import static org.assertj.core.api.BDDAssertions.*;


@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.DEFINED_PORT)
public class AddNewAssignmentToRosterServiceSystemTest {

    @Autowired
    private TestRestTemplate restTemplate;

    @Autowired
    private AddRosterItemPort addRosterItemPort;

    @Test
    void addNewAssignmentToRosterService() throws JSONException {

        String taskId = "TEST-ID";
        String executorType = "TEST-TYPE";
        String inputData = "TEST-DATA";
        String executorURI = "TEST-URI";

        ResponseEntity response = whenAddNewAssignmentToRoster(taskId, executorType, inputData, executorURI);

        JSONObject responseJson = new JSONObject(response.getBody().toString());
        String respTaskId = responseJson.getString("taskID");
        String respTaskType = responseJson.getJSONObject("taskType").getString("value");
        String respInputData = responseJson.getString("inputData");

        then(response.getStatusCode()).isEqualTo(HttpStatus.OK);
        then(respTaskId).isEqualTo(respTaskId);
        then(respTaskType).isEqualTo(executorType);
        then(respInputData).isEqualTo(inputData);
        then(Roster.getInstance().getRosterMap().size()).isEqualTo(1);
    }

    private ResponseEntity whenAddNewAssignmentToRoster(
        String taskId,
        String executorType,
        String inputData,
        String executorURI) throws JSONException {

         Roster roster = Roster.getInstance();
         roster.getRosterMap().clear();
         roster.addTaskToQueue(new Task(taskId, new ExecutorType(executorType), inputData));


        HttpHeaders headers = new HttpHeaders();
        headers.add("Content-Type", "application/json");

        String jsonPayLoad = new JSONObject()
            .put("executorType", executorType)
            .put("executorURI", executorURI)
            .toString();

        HttpEntity<String> request = new HttpEntity<>(jsonPayLoad, headers);

        return restTemplate.exchange(
            "/task/apply/",
            HttpMethod.POST,
            request,
            Object.class
        );
    }



}
