package ch.unisg.tapas.auctionhouse.adapter.in.messaging.mqtt;

import ch.unisg.tapas.auctionhouse.application.handler.AuctionStartedHandler;
import ch.unisg.tapas.auctionhouse.application.port.in.AuctionStartedEvent;
import ch.unisg.tapas.auctionhouse.domain.Auction;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.eclipse.paho.client.mqttv3.MqttMessage;

import java.net.URI;
import java.sql.Timestamp;

public class ExternalAuctionStartedEventListenerMqttAdapter extends AuctionEventMqttListener{
    private static final Logger LOGGER = LogManager.getLogger(ExternalAuctionStartedEventListenerMqttAdapter.class);

    @Override
    public void handleEvent(MqttMessage message){
        String payload = new String(message.getPayload());

        System.out.println("New external MQTT auction");

        try {
            // Note: this message representation is provided only as an example. You should use a
            // representation that makes sense in the context of your application.
            JsonNode data = new ObjectMapper().readTree(payload);

            // TODO Sanitize URIs
            String auctionId = data.get("auctionId").asText();
            String auctionHouseUri = data.get("auctionHouseUri").asText();
            String taskUri = data.get("taskUri").asText();
            String taskType = data.get("taskType").asText();
            String deadline = data.get("deadline").asText();

            var auction = new Auction(
                new Auction.AuctionId(auctionId),
                new Auction.AuctionHouseUri(URI.create(auctionHouseUri)),
                new Auction.AuctionedTaskUri(URI.create(taskUri)),
                new Auction.AuctionedTaskType(taskType),
                new Auction.AuctionDeadline(Timestamp.valueOf(deadline))
            );

            var event = new AuctionStartedEvent(auction);
            var handler = new AuctionStartedHandler();
            handler.handleAuctionStartedEvent(event);


    } catch (JsonProcessingException e) {
            e.printStackTrace();
        }
    }
}
