package ch.unisg.tapas.auctionhouse.adapter.in.messaging.websub;

import ch.unisg.tapas.auctionhouse.application.handler.AuctionStartedHandler;
import ch.unisg.tapas.auctionhouse.application.port.in.AuctionStartedEvent;
import ch.unisg.tapas.auctionhouse.domain.Auction;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import java.net.URI;
import java.net.URISyntaxException;
import java.sql.Timestamp;

/**
 * This class is a template for handling auction started events received via WebSub
 */
@RestController
public class ExternalAuctionStartedEventListenerWebSubAdapter {
    private final AuctionStartedHandler auctionStartedHandler;

    public ExternalAuctionStartedEventListenerWebSubAdapter(AuctionStartedHandler auctionStartedHandler) {
        this.auctionStartedHandler = auctionStartedHandler;
    }
    /**
    *   Controller which listens to auction-started callbacks
    *   @return 200 OK
     * @throws URISyntaxException
    **/
    // TODO generate a new capability ID instead of using a hardcoded one.
    @PostMapping(path = "/auction-started/74c72c7f-2739-4124-aa35-a3225171a97c")
    public ResponseEntity<Void> handleExecutorAddedEvent(@RequestBody String payload) throws URISyntaxException {

        System.out.println("New external WebSub auction");
        System.out.println(payload);


        JSONArray auctions = new JSONArray(payload);
        if (auctions.length() > 0) {
            JSONObject auctionJson = auctions.getJSONObject(0);
            System.out.print(auctionJson);

            String auctionId = auctionJson.getString("auctionId");
            String auctionHouseUri = auctionJson.getString("auctionHouseUri");
            String taskUri = auctionJson.getString("taskUri");
            String taskType = auctionJson.getString("taskType");
            String deadline = auctionJson.getString("deadline");

            var auction = new Auction(
                new Auction.AuctionId(auctionId),
                new Auction.AuctionHouseUri(URI.create(auctionHouseUri)),
                new Auction.AuctionedTaskUri(URI.create(taskUri)),
                new Auction.AuctionedTaskType(taskType),
                new Auction.AuctionDeadline(Timestamp.valueOf(deadline))
            );

            var event = new AuctionStartedEvent(auction);
            auctionStartedHandler.handleAuctionStartedEvent(event);
        }

        return new ResponseEntity<>(HttpStatus.OK);
    }
}
