package ch.unisg.tapastasks.tasks.application.handler;

import ch.unisg.tapastasks.tasks.application.port.in.TaskAssignedEvent;
import ch.unisg.tapastasks.tasks.application.port.in.TaskAssignedEventHandler;
import ch.unisg.tapastasks.tasks.application.port.out.AddTaskPort;
import ch.unisg.tapastasks.tasks.application.port.out.LoadTaskPort;
import ch.unisg.tapastasks.tasks.domain.Task;
import ch.unisg.tapastasks.tasks.domain.TaskNotFoundException;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import javax.transaction.Transactional;
import java.util.Optional;

@RequiredArgsConstructor
@Component
@Transactional
public class TaskAssignedHandler implements TaskAssignedEventHandler {
    // TODO Why do we need this event handler when a service exists that does the same?
    private final AddTaskPort addTaskToRepositoryPort;
    private final LoadTaskPort loadTaskFromRepositoryPort;

    @Override
    public Task handleTaskAssigned(TaskAssignedEvent taskAssignedEvent) throws TaskNotFoundException {
        // retrieve the task based on ID
        Optional<Task> taskFromRepo = Optional.ofNullable(loadTaskFromRepositoryPort.loadTask(taskAssignedEvent.getTaskId()));

        // update the status to assigned
        Task updatedTask = taskFromRepo.get();
        updatedTask.setTaskStatus(new Task.TaskStatus(Task.Status.ASSIGNED));

        // save updated task in repo
        addTaskToRepositoryPort.addTask(updatedTask);

        return updatedTask;
    }
}
