package ch.unisg.tapastasks.tasks.application.handler;

import ch.unisg.tapastasks.tasks.application.port.in.TaskExecutedEvent;
import ch.unisg.tapastasks.tasks.application.port.in.TaskExecutedEventHandler;
import ch.unisg.tapastasks.tasks.application.port.out.AddTaskPort;
import ch.unisg.tapastasks.tasks.application.port.out.LoadTaskPort;
import ch.unisg.tapastasks.tasks.domain.Task;
import ch.unisg.tapastasks.tasks.domain.TaskNotFoundException;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import javax.transaction.Transactional;
import java.util.Optional;

@RequiredArgsConstructor
@Component
@Transactional
public class TaskExecutedHandler implements TaskExecutedEventHandler {

    private final AddTaskPort addTaskToRepositoryPort;
    private final LoadTaskPort loadTaskFromRepositoryPort;

    @Override
    public Task handleTaskExecuted(TaskExecutedEvent taskExecutedEvent) throws TaskNotFoundException {
        // retrieve the task based on ID
        Optional<Task> taskFromRepo = Optional.ofNullable(loadTaskFromRepositoryPort.loadTask(taskExecutedEvent.getTaskId()));

        // update the status to assigned
        Task updatedTask = taskFromRepo.get();
        updatedTask.setTaskStatus(new Task.TaskStatus(Task.Status.EXECUTED));

        if(taskExecutedEvent.getServiceProvider().isPresent()){
            updatedTask.setProvider(taskExecutedEvent.getServiceProvider().get());
        }

        if(taskExecutedEvent.getOutputData().isPresent()){
            updatedTask.setOutputData(taskExecutedEvent.getOutputData().get());
        }

        // save updated task in repo
        addTaskToRepositoryPort.addTask(updatedTask);

        return updatedTask;
    }
}
