package ch.unisg.tapastasks.tasks.application.handler;

import ch.unisg.tapastasks.tasks.application.port.in.TaskStartedEvent;
import ch.unisg.tapastasks.tasks.application.port.in.TaskStartedEventHandler;
import ch.unisg.tapastasks.tasks.application.port.out.AddTaskPort;
import ch.unisg.tapastasks.tasks.application.port.out.LoadTaskPort;
import ch.unisg.tapastasks.tasks.domain.Task;
import ch.unisg.tapastasks.tasks.domain.TaskNotFoundException;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import javax.transaction.Transactional;
import java.util.Optional;

@RequiredArgsConstructor
@Component
@Transactional
public class TaskStartedHandler implements TaskStartedEventHandler {

    private final AddTaskPort addTaskToRepositoryPort;
    private final LoadTaskPort loadTaskFromRepositoryPort;

    @Override
    public Task handleTaskStarted(TaskStartedEvent taskStartedEvent) throws TaskNotFoundException {
        // retrieve the task based on ID
        Optional<Task> taskFromRepo = Optional.ofNullable(loadTaskFromRepositoryPort.loadTask(taskStartedEvent.getTaskId()));

        // update the status to assigned
        Task updatedTask = taskFromRepo.get();
        updatedTask.setTaskStatus(new Task.TaskStatus(Task.Status.RUNNING));

        if(taskStartedEvent.getServiceProvider().isPresent()){
            updatedTask.setProvider(taskStartedEvent.getServiceProvider().get());
        }

        // save updated task in repo
        addTaskToRepositoryPort.addTask(updatedTask);

        return updatedTask;
    }
}
