package ch.unisg.tapastasks.tasks.application.service;

import ch.unisg.tapastasks.tasks.application.port.in.CompleteTaskCommand;
import ch.unisg.tapastasks.tasks.application.port.in.CompleteTaskUseCase;
import ch.unisg.tapastasks.tasks.application.port.out.AddTaskPort;
import ch.unisg.tapastasks.tasks.application.port.out.ExternalTaskExecutedEvent;
import ch.unisg.tapastasks.tasks.application.port.out.ExternalTaskExecutedEventHandler;
import ch.unisg.tapastasks.tasks.application.port.out.LoadTaskPort;
import ch.unisg.tapastasks.tasks.domain.Task;
import ch.unisg.tapastasks.tasks.domain.Task.*;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import javax.transaction.Transactional;
import java.util.Optional;

@RequiredArgsConstructor
@Component
@Transactional
public class CompleteTaskService implements CompleteTaskUseCase {

    private final AddTaskPort addTaskToRepositoryPort;
    private final LoadTaskPort loadTaskFromRepositoryPort;
    private final ExternalTaskExecutedEventHandler externalTaskExecutedEventHandler;

    @Override
    public Task completeTask(CompleteTaskCommand command){
        // retrieve the task based on ID
        Optional<Task> taskFromRepo = Optional.ofNullable(loadTaskFromRepositoryPort.loadTask(command.getTaskId()));

        Task updatedTask = taskFromRepo.get();
        updatedTask.setOutputData(command.getOutputData());
        updatedTask.setTaskStatus(new TaskStatus(Task.Status.EXECUTED));

        // save updated task in repo
        addTaskToRepositoryPort.addTask(updatedTask);

        if (updatedTask.getOriginalTaskUri() != null) {
            ExternalTaskExecutedEvent event = new ExternalTaskExecutedEvent(
                updatedTask.getTaskId(),
                updatedTask.getOriginalTaskUri(),
                updatedTask.getOutputData()
                );
            externalTaskExecutedEventHandler.handleEvent(event);
        }

        return updatedTask;
    }
}
