package ch.unisg.tapastasks.tasks.application.service;


import ch.unisg.tapastasks.tasks.application.port.in.DeleteTaskCommand;
import ch.unisg.tapastasks.tasks.application.port.in.DeleteTaskUseCase;
import ch.unisg.tapastasks.tasks.application.port.out.AddTaskPort;
import ch.unisg.tapastasks.tasks.application.port.out.CanTaskBeDeletedPort;
import ch.unisg.tapastasks.tasks.application.port.out.DeleteTaskPort;
import ch.unisg.tapastasks.tasks.application.port.out.LoadTaskPort;
import ch.unisg.tapastasks.tasks.domain.DeleteTaskEvent;
import ch.unisg.tapastasks.tasks.domain.Task;
import jdk.jshell.spi.ExecutionControl;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import javax.transaction.Transactional;
import java.util.Optional;

@RequiredArgsConstructor
@Component
@Transactional
public class DeleteTaskService implements DeleteTaskUseCase {

    private final DeleteTaskPort deleteTaskFromRepositoryPort;
    private final LoadTaskPort loadTaskFromRepositoryPort;
    private final CanTaskBeDeletedPort canTaskBeDeletedPort;

    @Override
    public Optional<Task> deleteTask(DeleteTaskCommand command){
        // retrieve the task based on ID
        Optional<Task> taskFromRepo = Optional.ofNullable(loadTaskFromRepositoryPort.loadTask(command.getTaskId()));

        if(taskFromRepo.isPresent() && taskFromRepo.get().getTaskStatus().getValue().equals(Task.Status.OPEN)){
            // If task exists, and it is open then we try deleting it from the roster
            if(canTaskBeDeletedPort.canTaskBeDeletedEvent(new DeleteTaskEvent(command.getTaskId().getValue()))){
                deleteTaskFromRepositoryPort.deleteTask(taskFromRepo.get().getTaskId());
                return taskFromRepo;
            }
        }

        return Optional.empty();
    }
}
