package ch.unisg.tapastasks.tasks.application.service;

import ch.unisg.tapastasks.tasks.application.port.in.TaskAssignedCommand;
import ch.unisg.tapastasks.tasks.application.port.in.TaskAssignedUseCase;
import ch.unisg.tapastasks.tasks.application.port.out.AddTaskPort;
import ch.unisg.tapastasks.tasks.application.port.out.LoadTaskPort;
import ch.unisg.tapastasks.tasks.domain.Task;
import ch.unisg.tapastasks.tasks.domain.Task.*;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import javax.transaction.Transactional;
import java.util.Optional;

@RequiredArgsConstructor
@Component
@Transactional
public class TaskAssignedService implements TaskAssignedUseCase {

    private final AddTaskPort addTaskToRepositoryPort;
    private final LoadTaskPort loadTaskFromRepositoryPort;

    @Override
    public Task assignTask(TaskAssignedCommand command) {
        // retrieve the task based on ID
        Optional<Task> taskFromRepo = Optional.ofNullable(loadTaskFromRepositoryPort.loadTask(command.getTaskId()));

        // update the status to assigned
        Task updatedTask = taskFromRepo.get();
        updatedTask.setTaskStatus(new TaskStatus(Status.ASSIGNED));

        // save updated task in repo
        addTaskToRepositoryPort.addTask(updatedTask);

        return updatedTask;
    }
}
